@@ -5,7 +5,7 @@ from skimage.transform import (AffineTransform, EuclideanTransform,
                                SimilarityTransform)
 
 from . import defaults, libcoordio
-from .exceptions import CoordIOUserWarning
+from .exceptions import CoordIOUserWarning, CoordIOError
 
 
 # low level, functional conversions
@@ -474,7 +474,8 @@ def focalToField(xFocal, yFocal, zFocal, site, waveCat):
 
 def focalToWok(
     xFocal, yFocal, zFocal, positionAngle=0,
-    xOffset=0, yOffset=0, zOffset=0, tiltX=0, tiltY=0
+    xOffset=0, yOffset=0, zOffset=0, tiltX=0, tiltY=0,
+    fpScale=defaults.FOCAL_SCALE, projectFlat=True
 ):
     """Convert xyz focal coordinates in mm to xyz wok coordinates in mm.
 
@@ -485,6 +486,8 @@ def focalToWok(
 
     Tilt is applied about x axis then y axis.
 
+    Note: currently zFocal irrelevant using a flat wok model.
+
 
     Parameters
     -------------
@@ -517,6 +520,10 @@ def focalToWok(
     tiltY: scalar
         tilt (deg) of wok about focal y axis at PA=0
         calibrated
+    fpScale: scalar
+        radial scale multiplier
+    projectFlat: bool
+        if true, assume flat wok model
 
     Returns
     ---------
@@ -584,12 +591,29 @@ def focalToWok(
         # single set of xyz coords fed in
         xWok, yWok, zWok = coords - transXYZ
 
+    # scale xyWok from dither analysis results
+    xWok = xWok / fpScale # from gfa: 0.9998899
+    yWok = yWok / fpScale # from gfa: 0.9998899
+
+    if projectFlat:
+        # force all z coords to be positioner height
+        # necessary for a flat wok model
+        # and this should be removed if we go
+        # to curved wok
+        # !!!!!!!!!!!!!!!!!
+        if hasattr(xFocal, "__len__"):
+            zWok = numpy.array([defaults.POSITIONER_HEIGHT]*len(xFocal)).T
+        else:
+            zWok = defaults.POSITIONER_HEIGHT
+
     return xWok, yWok, zWok
 
 
 def wokToFocal(
     xWok, yWok, zWok, positionAngle=0,
-    xOffset=0, yOffset=0, zOffset=0, tiltX=0, tiltY=0
+    xOffset=0, yOffset=0, zOffset=0, tiltX=0, tiltY=0,
+    fpScale=defaults.FOCAL_SCALE,
+    b=None, R=None
 ):
     """Convert xyz wok coordinates in mm to xyz focal coordinates in mm.
 
@@ -600,6 +624,8 @@ def wokToFocal(
 
     Tilt is applied first about x axis then y axis.
 
+    Note: currently zWok is irrelevant using a flat wok model.
+
 
     Parameters
     -------------
@@ -630,6 +656,13 @@ def wokToFocal(
     tiltY: scalar
         tilt (deg) of wok about focal y axis at PA=0
         calibrated
+    fpScale: scalar
+        radial scale multiplier
+    b: float
+        z location of the center of curvature, dist from M1 vertex (mm)
+    R: float
+        radius of curvature (mm)
+
 
     Returns
     ---------
@@ -643,6 +676,11 @@ def wokToFocal(
         z position of object in focal coord sys mm
         (+z aligned boresight and increases from the telescope to the sky)
     """
+
+    # scale xyWok from dither analysis results
+    xWok = numpy.array(xWok) * fpScale # from gfa: 0.9998899
+    yWok = numpy.array(yWok) * fpScale # from gfa: 0.9998899
+
     # this routine is a reversal of the steps
     # in the function focalToWok, with rotational
     # angles inverted and translations applied in reverse
@@ -684,6 +722,20 @@ def wokToFocal(
     coords = rotMatZ.dot(coords)
     coords = rotMatY.dot(coords)
     xFocal, yFocal, zFocal = rotMatX.dot(coords)
+
+    if b is not None or R is not None:
+        # calculate z coords based on xy
+        # basically force the point to land on the focal
+        # surface for a given wavelength
+        # necessary for a flat wok model
+        # and this should be removed if we go
+        # to curved wok
+        # !!!!!!!!!!!!!!!!!
+        if R is None or b is None:
+            raise CoordIOError("Must specify both b and R if they are not None")
+        rFocal = numpy.sqrt(xFocal**2+yFocal**2)
+        zFocal = -1*numpy.sqrt(R**2-rFocal**2) + b
+
     return xFocal, yFocal, zFocal
 
 
